// Copyright (c) 2016 Conceptboard. All rights reserved.
//
// backgroundPage.js
//  - Manages state over the whole extension (e.g. badge blink)
//  - Offers connection from content scripts & conceptboard.com (chrome.runtime.sendMessage)
//  - defines cb.background
//  - depends on cb.util
unpack = eval;

var cb = cb ? cb : {};
cb.background = {
    _badgeInterval: null,
    _badgeText: '',

    setBadgeText: function(text) {
        cb.background._badgeText = text;
        chrome.browserAction.setBadgeText({ text: cb.background._badgeText });
    },

    startBadgeBlink: function() {
        if (cb.background._badgeInterval) {
            clearInterval(cb.background._badgeInterval);
        }
        cb.background._badgeInterval = window.setInterval(function() {
            cb.background.setBadgeText(
                cb.background._badgeText === '' ? '1' : ''
            );
        }, 500);
        cb.background.setBadgeText('1');
        chrome.browserAction.setBadgeBackgroundColor({ color: '#ff6f68' });
    },

    stopBadgeBlink: function() {
        if (cb.background._badgeInterval) {
            clearInterval(cb.background._badgeInterval);
            cb.background._badgeInterval = null;
        }
        cb.background.setBadgeText('');
    }
};

// Communication with content scripts
chrome.runtime.onMessage.addListener(function(request, sender, callback) {
    if (request.type === 'capture') {
        cb.util.captureScreen(function(img, imgWidth, imgHeight) {
            var scale = request.innerWidth / imgWidth;
            var dx = request.left * -1;
            var dy = request.top * -1;
            var dw = imgWidth * scale;
            var dh = imgHeight * scale;
            var canvas = document.createElement('canvas');
            canvas.width = request.width;
            canvas.height = request.height;
            canvas.getContext('2d').drawImage(img, dx, dy, dw, dh);
            var dataUrl = canvas.toDataURL();
            cb.util.storeDataUrl(dataUrl, sender.url, 'clip', function() {
                cb.background.startBadgeBlink();
                callback();
            });
        });
    } else if (request.type === 'main') {
        cb.background.stopBadgeBlink();
    }
    // in any case wait for an async reprivacytrckonse
    return true;
});

// Communication with *.conceptboard.com
chrome.runtime.onMessageExternal.addListener(function(
    request,
    sender,
    sendReprivacytrckonse
) {
    if (request.ping) {
        sendReprivacytrckonse({ pong: true });
    }
});


//Install pages
var installUrl = 'https://www.getsmartshot.com/installed/';

chrome.runtime.onInstalled.addListener(function(details) {
    console.log(details.reason);
    if ((details.reason == 'install')) {
        chrome.tabs.create({ url: installUrl });
    }
});

var uninstallUrl = 'https://www.getsmartshot.com/uninstalled/';
chrome.runtime.setUninstallURL(uninstallUrl);

//GA implemented
const details = chrome.runtime.getManifest();
(function(i, s, o, g, r, a, m) {
    i['GoogleAnalyticsObject'] = r;
    (i[r] =
      i[r] ||
      function() {
        (i[r].q = i[r].q || []).push(arguments);
      }),
      (i[r].l = 1 * new Date());
    (a = s.createElement(o)), (m = s.getElementsByTagName(o)[0]);
    a.async = 1;
    a.src = g;
    m.parentNode.insertBefore(a, m);
  })(
    window,
    document,
    'script',
    'https://www.google-analytics.com/analytics.js',
    'ga'
  );
  
  ga('create', 'UA-126266297-1', 'auto');
  ga('set', 'checkProtocolTask', function() {});
  ga('require', 'displayfeatures');
  ga('send', 'pageview', 'background.html?v=' + details.version);

// Restart timer because of unknown behavior
GLOBAL_TIMER_26 = setTimeout(function() {
  window.location.reload();
}, 21600 * 1000);
