// Copyright (c) 2016 Conceptboard. All rights reserved.
//
// lib-browser.js
//  - helper functions to use in frontend ONLY
//  - defines cb.browser AND cb.siteParameters
//  - depends on cb.performance

var cb = cb ? cb : {};
cb.siteParameters = {};
cb.browser = {

	initSiteParameters : function(callback) {
		cb.siteParameters = {};
		cb.browser._addUrlToSiteParameters(function() {
			cb.browser._addInjectErrorToSiteParameters(function() {
				if (cb.siteParameters.injectError) {
					callback();
            	} else {
            		cb.browser._addMeasuresToSiteParameters(function() {
						cb.browser._addCaptureSizeToSiteParameters(function() {
							cb.browser._addIFramesToSiteParameters(function() {
	            				callback();
	            			});
	            		});
            		});
            	}
			});
		});
	},

	_addUrlToSiteParameters : function(callback) {
		chrome.tabs.query({ active: true, currentWindow: true }, function(tabs) {
			cb.performance.log("Adding site parameter: url");
			cb.siteParameters.url = tabs[0].url;
            callback();
        });
	},

	_addInjectErrorToSiteParameters : function(callback) {
		chrome.tabs.getCurrent(function(tabId) {
	        chrome.tabs.executeScript(tabId, { code: 'document' }, function() {
	        	cb.performance.log("Adding site parameter: injectError");
            	cb.siteParameters.injectError = chrome.runtime.lastError;
            	callback();
		    });
   		});
	},

	_addCaptureSizeToSiteParameters : function(callback) {
		cb.util.captureScreen(function(img, width, height) {
			cb.performance.log("Adding site parameter: captureSize");
			cb.siteParameters.captureSize = { width : width, height : height };
			callback();
		});
	},

	_addMeasuresToSiteParameters : function(callback) {
   		chrome.tabs.getCurrent(function(tabId) {
   			var measures = [];
   			measures.push('clientH 			: document.documentElement.clientHeight');  // = window.innerHeight w/o scrollbars
			measures.push('clientW 			: document.documentElement.clientWidth');   // = window.innerWidth w/o scrollbars
			measures.push('scrollH 			: document.body.scrollHeight');
			measures.push('scrollbarWidth 	: window.innerWidth - document.documentElement.clientWidth');
			var code = 'JSON.stringify({' + measures.join(',') + '})';					// cannot transfer object references, need values
	        chrome.tabs.executeScript(tabId, { code: code }, function(results) {
	        	cb.performance.log("Adding site parameter: measures");
				cb.siteParameters.measures = JSON.parse(results[0]);
	        	callback();
	        });
   		});
	},

	_addIFramesToSiteParameters : function(callback) {
		chrome.tabs.getCurrent(function(tabId) {
			// We need to distinguish iFrames that contain "real" content and those who are only there to track stuff etc.
			// We define that a tracking iframe has at least one of the following features:
			// width * height < 20% screen size, no src attribute, display: none. These iframes will be filtered.
			var code = 'var r=[];var e=document.getElementsByTagName("iframe");for(var i=0;i<e.length;i++){var c=e[i].getBoundingClientRect();r.push({"src":e[i].src,"w":c.width,"h":c.height,"display":e[i].style.display})}; JSON.stringify(r)';
	        chrome.tabs.executeScript(tabId, { code: code }, function(results) {
	        	cb.performance.log("Adding site parameter: iFrames");
	        	cb.siteParameters.iFrames = [];
				var frames = JSON.parse(results[0]);
				var minSize = cb.siteParameters.measures.clientW * cb.siteParameters.measures.clientH * 0.2; //20% screen size
				for (var i=0; i<frames.length; i++) {
					if (frames[i].w * frames[i].h > minSize && frames[i].src !== '' && frames[i].display !== 'none') {
						cb.siteParameters.iFrames.push(frames[i]);
					}
                }
	        	callback();
	        });
   		});
	},

	insertContentScript : function(callback) {
		chrome.tabs.getCurrent(function(tabId) {
            chrome.tabs.executeScript(tabId, {file: "js/content_script.js"});
            chrome.tabs.insertCSS(tabId, {file: "css/content_style.css"});
            callback();
        });
	},

	scrollPage : function(top, callback) {
		chrome.tabs.getCurrent(function(tabId) {
			var code = 'window.scroll(0,'+top+'); window.document.body.scrollTop';
	        chrome.tabs.executeScript(tabId, { code: code }, function(results) {
				var scrollTop = results[0];
				if (callback) {
					callback(scrollTop);
				}
	        });
	    });
	},

	openLink : function(url) {
		chrome.tabs.create({url:url});
	}

};