// Copyright (c) 2016 Conceptboard. All rights reserved.
//
// lib-capture-fullpage.js
//  - helper functions to use in frontend ONLY
//  - defines cb.captureFullPage
//  - depends on cb.config, cb.browser, cb.siteParameters

var cb = cb ? cb : {};
cb.captureFullPage = {

    start : function(progressCallback, finalCallback) {
        // Define new process data object
        var processData = {};
        // Store callbacks for later
        processData.progressCallback = progressCallback;
        processData.finalCallback = finalCallback;
        // Creating a new canvas for the result in the "logical" resolution (JS coordinates)
        processData.canvas = document.createElement('canvas');
        processData.canvas.width = cb.siteParameters.measures.clientW;
        processData.canvas.height = cb.siteParameters.measures.scrollH;
        // The actual image width of the screenshots may be bigger than the "logical" size (see devicePixelRatio)
        // But on Chrome device simulation, it may be even "zoomed", so the devicePixelRatio is actually wrong... :/
        // When comparing the clientW with the capturedSize, we need to take into account that the scrollbar is captured, too.
        processData.scale = (cb.siteParameters.measures.clientW + cb.siteParameters.measures.scrollbarWidth) / cb.siteParameters.captureSize.width;
        // In general we want to scroll in steps of a "client height" minus the offsets on top / bottom ("logical" resolution)
        processData.offset = Math.round(cb.config.staticRatio * cb.siteParameters.measures.clientH);
        processData.scrollDelay = cb.config.scrollDelay;
        // We're remembering a "last scroll top" so we can exit even in case the page "shinks" during our process
        processData.lastScrollTop = -1;
        // Go...
        processData.progressCallback(0);
        cb.captureFullPage._nextCapture(processData, 0);
    },

    _insertCapture : function(processData, img, scrollTop) {
        var isFirstFrame = scrollTop === 0;
        var isLastFrame = scrollTop + cb.siteParameters.measures.clientH >= cb.siteParameters.measures.scrollH;
        var scrollbarWidth = cb.siteParameters.measures.scrollbarWidth;
        if (isFirstFrame) {
            // 1st frame: Taking the complete image, destination width/height equals "logical" resolution
            var dw = img.width * processData.scale;
            var dh = img.height * processData.scale;
            processData.canvas.getContext('2d').drawImage(img, scrollbarWidth, 0, dw, dh);
        } else {
            // All later frames: only take everything below the top offset, the bottom offset is ignored and will always be overwritten.
            // Source sizes equal the actual captured image (not "logical"), the ("logical") offset on top needs to get converted.
            var sw = img.width;
            var sh = img.height - processData.offset / processData.scale;
            var sy = processData.offset / processData.scale;
            // Destination size in "logical" resolution, height is lower as we substract the ("logical") offset on top.
            var dw = img.width * processData.scale;
            var dh = img.height * processData.scale - processData.offset;
            // For the last image, we overwrite everything that's already at the bottom
            var dy = isLastFrame ? (cb.siteParameters.measures.scrollH - dh) : (scrollTop + processData.offset);
            processData.canvas.getContext('2d').drawImage(img, 0, sy, sw, sh, scrollbarWidth, dy, dw, dh);
        }
        return isLastFrame;
    },

    _nextCapture : function(processData, nextScrollTop) {
        var progressInPercent = Math.min(100, Math.round(nextScrollTop / cb.siteParameters.measures.scrollH * 100));
        processData.progressCallback(progressInPercent);
        cb.browser.scrollPage(nextScrollTop, function(scrollTop) {
            console.debug("Scrolling executed:", scrollTop);
            setTimeout(function() {
                cb.util.captureScreen(function(img, width, height) {
                    console.debug("Screenshot executed:", scrollTop, width, height);
                    var isLastFrame = cb.captureFullPage._insertCapture(processData, img, scrollTop);
                    var scrollBlocked = processData.lastScrollTop === scrollTop;
                    if (!scrollBlocked && !isLastFrame) {
                        processData.lastScrollTop = scrollTop;
                        var nextScrollTop = scrollTop + cb.siteParameters.measures.clientH - (2 * processData.offset);
                        cb.captureFullPage._nextCapture(processData, nextScrollTop);
                    } else {
                        cb.captureFullPage._end(processData);
                    }
                });
            }, processData.scrollDelay);
        });
    },

    _end : function(processData) {
        processData.progressCallback(100);
        cb.browser.scrollPage(0);
        var dataUrl = processData.canvas.toDataURL();
        cb.util.storeDataUrl(dataUrl, cb.siteParameters.url, 'full', function() {
            processData.finalCallback();
        });
    }

};


