// Copyright (c) 2016 Conceptboard. All rights reserved.
//
// Great sites for full screen testing:
// - Super long page:               http://zeit.de  (works!)
// - Weird header and shrinking:    http://www.sitepoint.com/jquery-settimeout-function-examples/
// - Static footer:                 http://www.golem.de/ (before accepting the cookies)
// - Weird header + footer:         http://www.theguardian.com/ (before accepting the cookies)
//

var cb = cb ? cb : {};
cb.config = {};
cb.defaultConfig = {
  scrollDelay: '200', // time the screenshot waits after scrolling
  staticRatio: '0.2' // static headers/footers up to 20% height are ignored
};
cb.main = {
  onDomLoaded: function() {
    // sync config...
    cb.storage.syncConfigWithStore();
    cb.performance.log('Config synced with local storage');
    // initialize UI with event handlers...
    cb.main.initUi();
    cb.performance.log('UI events initialized');
    // load existing images...
    cb.main.refreshUi();
    cb.performance.log('Refresh UI started');
    // init site specific parameters...
    cb.browser.initSiteParameters(function(siteParameters) {
      if (cb.siteParameters.injectError) {
        cb.main.disableCaptureUi(cb.siteParameters.url);
        console.warn(cb.siteParameters.injectError.message);
      }
      if (cb.siteParameters.iFrames) {
        cb.main.populateIFramesList(cb.siteParameters.iFrames);
      }
      cb.performance.log('Init site parameters finished');
    });
    // telling the background page that we've loaded
    chrome.runtime.sendMessage({ type: 'main' });
  },

  performFullPageCapture: function() {
    cb.captureFullPage.start(
      function(progressInPercent) {
        $('#progress-number').html(progressInPercent + '%');
        $('#fullpage_progress_bar').css('width', progressInPercent + '%');
      },
      function() {
        window.location.reload();
      }
    );
  },

  clickCaptureFullPage: function() {
    if (cb.siteParameters.iFrames && cb.siteParameters.iFrames.length > 0) {
      $('#main_screen, #iframes_screen').toggle();
    } else {
      $('#main_screen, #fullpage_screen').toggle();
      if (cb.main._validateConfig()) {
        cb.main.performFullPageCapture();
      }
    }
  },

  clickSkipIframes: function() {
    $('#iframes_screen, #fullpage_screen').toggle();
    cb.main.performFullPageCapture();
  },

  clickCaptureSnippet: function() {
    cb.browser.insertContentScript(function() {
      window.close();
    });
  },

  initUi: function() {
    // make all links work
    $('a').on('click', function(e) {
      cb.browser.openLink(e.target.href);
    });

    // full page screenshot
    $('#fullpage_button').on('click', function() {
      cb.main.clickCaptureFullPage();
    });

    // screenshot with clip selection
    $('#clip_button').on('click', function() {
      cb.main.clickCaptureSnippet();
    });

    // skip iframe screen in full page process
    $('#skip_frames_button').on('click', function() {
      cb.main.clickSkipIframes();
    });

    // Meta-nav (tips, settings, library)
    $('.meta-nav li').on('click', function(evt) {
      $('#tips, #library, #settings').hide();
      if ($(this).hasClass('active')) {
        $('#headline').text('Library');
        $('.meta-nav li').removeClass('active');
        $('#library').show();
      } else {
        $('.meta-nav li').removeClass('active');
        $(this).addClass('active');
        var targetContent = $(this)
          .attr('id')
          .split('_')[0];
        $('#headline').text(targetContent);
        $('#' + targetContent).show();
      }
    });

    // setup tips slideshow
    $('#tips_show').append(
      cb.main._createSlideshowButtonNode('tips_show', false)
    );
    $('#tips_show').append(
      cb.main._createSlideshowButtonNode('tips_show', true)
    );
    $('#tips_show .slideshow-button')
      .last()
      .addClass('current');
    $('#tips_show').append(cb.main._createSlideshowPagingNode('tips_show'));

    // settings clear and restet functions
    $('[data-jsfunc="settings_storage_clear_all"]').on('click', function() {
      cb.storage.clearAllStorage(cb.main.refreshUi);
    });
    $('[data-jsfunc="settings_delay_reset"]').on('click', function() {
      $('#settings_delay_slider')[0].value = cb.defaultConfig.scrollDelay;
      cb.storage.storeConfigUpdate('scrollDelay', cb.defaultConfig.scrollDelay);
      cb.main.refreshUi(true);
    });
    $('[data-jsfunc="settings_ratio_reset"]').on('click', function() {
      $('#settings_ratio_slider')[0].value = cb.defaultConfig.staticRatio;
      cb.storage.storeConfigUpdate('staticRatio', cb.defaultConfig.staticRatio);
      cb.main.refreshUi(true);
    });

    // clear one specific item using attr data-key=timeInMs
    $('.slideshow').on(
      'click',
      '.slideshow-slide.current [data-jsfunc="settings_storage_clear"]',
      function(e) {
        cb.storage.clearItemFromStorage(
          e.currentTarget.getAttribute('data-itemkey'),
          cb.main.refreshUi
        );
      }
    );

    // changing scroll delay (and inital value)
    $('#settings_delay_slider').on('input', function(e) {
      $('#settings_delay_display').html(Math.round(e.target.value) + 'ms');
    });
    $('#settings_delay_slider')
      .val(cb.config.scrollDelay)
      .on('change', function(e) {
        cb.storage.storeConfigUpdate('scrollDelay', Math.round(e.target.value));
        cb.main.refreshUi(true);
      });
    // changing header/footer ratio (and inital value)
    $('#settings_ratio_slider').on('input', function(e) {
      $('#settings_ratio_display').html(Math.round(e.target.value * 100) + '%');
    });
    $('#settings_ratio_slider')
      .val(cb.config.staticRatio)
      .on('change', function(e) {
        cb.storage.storeConfigUpdate('staticRatio', e.target.value);
        cb.main.refreshUi(true);
      });
  },

  disableCaptureUi: function(siteUrl) {
    $('#main-nav').addClass('deactivated');
    if (siteUrl && siteUrl.indexOf('webstore') > -1) {
      $('#main-nav').addClass('deactivated-webstore');
    }
  },

  populateIFramesList: function(frames) {
    for (var i = 0; i < frames.length; i++) {
      var href = frames[i].src;
      var node = $(
        '<li><a>iFrame (' + frames[i].w + 'x' + frames[i].h + ')</a></li>'
      );
      node.on('click', function(e) {
        cb.browser.openLink(href);
      });
      $('#iframes_list').append(node);
    }
  },

  refreshUi: function(skipDataUrlUpdate) {
    console.debug('Refreshing UI', skipDataUrlUpdate);
    // re-loading all data may be skipped if there was no change
    if (!skipDataUrlUpdate) {
      cb.main._requestDataUrlContentRefresh();
    }
    // update setting sliders/displays
    $('#settings_delay_display').html(Math.round(cb.config.scrollDelay) + 'ms');
    $('#settings_ratio_display').html(
      Math.round(cb.config.staticRatio * 100) + '%'
    );
  },

  _dataUrlContentRefreshTimeout: null,
  _requestDataUrlContentRefresh: function() {
    console.debug(
      'Waiting to refresh data urls',
      cb.main._dataUrlContentRefreshTimeout
    );
    if (!cb.main._dataUrlContentRefreshTimeout) {
      cb.main._dataUrlContentRefreshTimeout = setTimeout(function() {
        cb.main._refreshDataUrlContent();
        cb.main._dataUrlContentRefreshTimeout = null;
      }, 100);
    }
  },

  _refreshDataUrlContent: function() {
    console.debug('Refreshing data urls');
    // retrieve screenshot from before and continue with the general 'images updated' method
    cb.storage.retrieveStoredDataUrls(cb.main._refreshLibraryUi);
    // update used storage in settings
    cb.storage.retrieveUsedDataUrlStorageInBytes(function(usedBytes) {
      var usedBytesString =
        Math.round((usedBytes / 1024 / 1024) * 10) / 10 + ' MB';
      $('#settings_storage_display').html(usedBytesString);
      cb.performance.log('Used storage display updated');
    });
  },

  _refreshLibraryUi: function(items) {
    try {
      // empty slideshow
      $('#library').empty();
      $('#library').append(
        cb.main._createSlideshowButtonNode('library', false)
      );
      $('#library').append(cb.main._createSlideshowButtonNode('library', true));
      // iterate over updated images
      var slides = 0;
      $.each(items, function(key, valueObj) {
        $('#library').append(cb.main._createSlideshowImageNode(key, valueObj));
        slides++;
      });
      $('#library').append(cb.main._createSlideshowPagingNode('library'));
      // update slideshow ui
      if (!slides) {
        $('#library').addClass('is-empty');
        $('#library').append(cb.main._createSlideshowEmptyNode());
      } else {
        $('#library').removeClass('is-empty');
        var firstSlide = $('#library .slideshow-slide').last();
        cb.main._loadImage(firstSlide);
        if (slides > 1) {
          $('#library .slideshow-button')
            .last()
            .addClass('current');
        }
      }
      cb.performance.log('Stored images display updated');
    } catch (e) {
      // in case something goes wrong, we try to show at least something to the user
      cb.main._showFatalError(e);
    }
  },

  _createSlideshowEmptyNode: function() {
    var emptyNodecontent = `
            <div class="empty-state">
                <div class="empty-state__content">   
                    <h2>No captures, yet</h2>
                    <p>Start by capturing a web page or by selecting a page snippet.</p>
                </div>
            </div>
            `;
    return $(emptyNodecontent);
  },

  _createSlideshowImageNode: function(timeInMs, valueObj) {
    var imageSrc = valueObj.dataUrl;
    var siteUrl = valueObj.siteUrl;
    var date = new Date(parseInt(timeInMs));
    var type = valueObj.type === 'full' ? 'Web Page' : 'Page Snippet';
    if (siteUrl.indexOf('://') > -1) {
      domain = siteUrl.split('/')[2];
    } else {
      domain = siteUrl.split('/')[0];
    }
    var truncatedUrl = siteUrl.replace(/https?:\/\//, '');
    var truncatedUrl = truncatedUrl.replace(/\/\s*$/, '');
    truncatedUrl =
      truncatedUrl.length > 21
        ? truncatedUrl.substring(0, 21) + '…'
        : truncatedUrl;
    var imageBlobUrl = cb.storage.dataUrlToBlob(imageSrc);
    var imgHead = 'data:image/png;base64,';
    var imgFileSize = Math.round(((imageSrc.length - imgHead.length) * 3) / 4);
    var imageMarkup =
      '<img src="img/blank_1x1.png" data-src="' + imageSrc + '"/>';
    var datestring =
      date.getFullYear() +
      '-' +
      ('0' + (date.getMonth() + 1)).slice(-2) +
      '-' +
      ('0' + date.getDate()).slice(-2) +
      '-' +
      ('0' + date.getHours()).slice(-2) +
      '-' +
      ('0' + date.getMinutes()).slice(-2);
    var downloadTitle = datestring + '-' + domain + '.png';
    var metaMarkup = `
            <div class="slideshow-slide__label">
                <h4>${type}</h4>
                <ul class="slideshow-slide__label__meta">
                    <li><a tabindex="-1" href="${siteUrl}" target="_blank" class="slideshow-slide__label__meta__url tooltip--color-white tooltip--size-small tooltip--break" data-tooltip="${siteUrl}">${truncatedUrl}</a></li>
                    <li>${date.toLocaleString()}</li>
                    <li><span class="js-img-width">width</span> × <span class="js-img-height">height</span> (${Math.round(
                      imgFileSize / 1024
                    )} KB)</li>
                    
                </ul><div class="slideshow-slide__label__meta__actions">
                        <a tabindex="-1" class="slideshow-slide__label__icon icon--download" href="${imageBlobUrl}" download="${downloadTitle}"><svg class="icon" style="position:relative; top: 1px; left: 2px;"><use xlink:href="#icon--download"></use></svg></a>
                        <span class="slideshow-slide__label__icon" data-jsfunc="settings_storage_clear" data-itemkey="${timeInMs}"><svg class="icon" style="position:relative; top: 1px; left: 2px;"><use xlink:href="#icon--trash-can"></use></svg></span>
                    </div>
            </div>
            `;
    return $(
      '<div class="slideshow-slide">' + imageMarkup + metaMarkup + '</div>'
    );
  },

  _createSlideshowButtonNode: function(slideshowId, isBack) {
    var innerMarkup = isBack
      ? '<svg class="icon icon--16_next"><use xlink:href="#icon--16_next"></use></svg>'
      : '<svg class="icon icon--16_prev"><use xlink:href="#icon--16_prev"></use></svg>';
    var isBackClass = isBack ? 'is-back' : '';
    var completeMarkup =
      '<div class="slideshow-button ' +
      isBackClass +
      '">' +
      innerMarkup +
      '</div>';
    return $(completeMarkup).on('click', function() {
      var oldNode = $('#' + slideshowId + ' .slideshow-slide.current');
      var newNode = isBack
        ? oldNode.prev('.slideshow-slide')
        : oldNode.next('.slideshow-slide');
      if (newNode.length > 0) {
        oldNode.removeClass('current');
        cb.main._loadImage(newNode);
        $('#' + slideshowId + ' .slideshow-button')
          .last()
          .toggleClass('current', newNode.prev('.slideshow-slide').length > 0);
        $('#' + slideshowId + ' .slideshow-button')
          .first()
          .toggleClass('current', newNode.next('.slideshow-slide').length > 0);
      }
      cb.main._updateCurrentSlideshowPosition(slideshowId);
    });
  },

  _createSlideshowPagingNode: function(slideshowId) {
    var total = $('#' + slideshowId + ' .slideshow-slide').length;
    var innerMarkup =
      '<span class="slideshow-pagination__current">1</span> / <span class="slideshow-pagination__all">' +
      total +
      '</span>';
    return (completeMarkup =
      '<div class="slideshow-pagination">' + innerMarkup + '</div>');
  },

  _updateCurrentSlideshowPosition: function(slideshowId) {
    var currentSlide = $('#' + slideshowId + ' .slideshow-slide.current')[0];
    var allSlides = $('#' + slideshowId + ' .slideshow-slide')
      .toArray()
      .reverse();
    var position = allSlides.indexOf(currentSlide) + 1;
    $('#' + slideshowId + ' .slideshow-pagination__current').html(position);
  },

  _loadImage: function(slideNode) {
    var imgNode = $(slideNode).children('img');
    $(slideNode).addClass('current');
    $(imgNode).addClass('loading');
    var origSrc = $(imgNode).attr('data-src');

    imgNode[0].addEventListener('load', function() {
      $(imgNode).removeClass('loading');
      $(imgNode)
        .siblings('.slideshow-slide__label')
        .find('.js-img-width')
        .html($(imgNode).prop('naturalWidth'));
      $(imgNode)
        .siblings('.slideshow-slide__label')
        .find('.js-img-height')
        .html($(imgNode).prop('naturalHeight'));
    });

    window.setTimeout(function() {
      $(imgNode).attr('src', origSrc);
    }, 100);
  },

  _validateConfig: function() {
    if (!cb.config) {
      console.error('Config object missing');
    } else if (
      cb.config.scrollDelay === undefined ||
      1 * cb.config.scrollDelay < 0
    ) {
      console.error('Config scroll delay must be a positive number');
    } else if (
      cb.config.staticRatio === undefined ||
      1 * cb.config.staticRatio < 0 ||
      1 * cb.config.staticRatio > 0.4
    ) {
      console.error('Config static ratio must be between 0 and 0.4');
    } else {
      return true;
    }
  },

  _showFatalError: function(e) {
    try {
      // Replacing the library content with an error message
      $('#library').empty();
      var msg =
        '<pre style="color:white; text-align:left; height:100%; overflow:hidden; margin:0;">';
      msg +=
        'FATAL ERROR:\nPlease try again or\nclear local storage (Settings).\n\n';
      msg +=
        'IF YOU ARE A NERD:\nPlease describe what you were doing\nand send it together with the data below\nto support@conceptboard.com. THX!\n\n';
      msg += e.name + '\n' + e.message + '\n' + e.stack;
      msg += '</pre>';
      $('#library').html(msg);
      // Switching to the library to make it visible
      $('#fullpage_screen, #iframes_screen, #settings, #tips').hide();
      $('#main_screen, #library').show();
    } catch (anotherError) {
      // Now everything is lost... we swallow this error to throw the original one
      console.error('Error showing error UI.');
    }
    throw e;
  }
};

document.addEventListener('DOMContentLoaded', cb.main.onDomLoaded);
